@@ -100,6 +100,7 @@ value first_names_brackets = ref None;
 value untreated_in_notes = ref False;
 value force = ref False;
 value default_source = ref "";
+value default_name = ref "?";
 value relation_status = ref Married;
 value no_picture = ref False;
 
@@ -510,7 +511,7 @@ type range 'a =
   [ Begin of 'a
   | End of 'a
   | BeginEnd of 'a and 'a ]
-; 
+;
 
 value date_g = Grammar.gcreate date_lexer;
 value date_value = Grammar.Entry.create date_g "date value";
@@ -525,6 +526,15 @@ value is_roman_int x =
   [ Not_found -> False ]
 ;
 
+value start_with_int x =
+  try
+    let s = String.sub x 0 1 in
+    let _ = int_of_string s in
+    True
+  with
+  [ _ -> False ]
+;
+
 value roman_int =
   let p =
     parser [: `("ID", x) when is_roman_int x :] -> Mutil.arabian_of_roman x
@@ -805,7 +815,7 @@ value add_string gen s =
         Hashtbl.add gen.g_hstr s (Adef.istr_of_int i);
         Adef.istr_of_int i
       } ]
-;        
+;
 
 value extract_addr addr =
   if String.length addr > 0 && addr.[0] = '@' then
@@ -910,9 +920,14 @@ value this_year =
   tm.Unix.tm_year + 1900
 ;
 
-value infer_death birth =
-  match birth with
-  [ Some (Dgreg d _) ->
+value infer_death birth bapt =
+  match (birth, bapt) with
+  [ (Some (Dgreg d _), _) ->
+      let a = this_year - d.year in
+      if a > dead_years.val then DeadDontKnowWhen
+      else if a <= alive_years.val then NotDead
+      else DontKnowIfDead
+  | (_, Some (Dgreg d _)) ->
       let a = this_year - d.year in
       if a > dead_years.val then DeadDontKnowWhen
       else if a <= alive_years.val then NotDead
@@ -920,6 +935,25 @@ value infer_death birth =
   | _ -> DontKnowIfDead ]
 ;
 
+(* Fonctions utiles pour la mise en forme des noms. *)
+
+(* Hashtbl (utf8.ml) qui font la correspondance entre : *)
+(*   - l'encoding -> le nom                             *)
+(*   - le nom     -> l'encoding                         *)
+value (ht_e_n, ht_n_e) =
+  do {
+    let ht_e_n = Hashtbl.create 5003 in
+    let ht_n_e = Hashtbl.create 5003 in
+    List.iter
+      (fun (encoding, name) ->
+        do {
+          Hashtbl.add ht_n_e name encoding;
+          Hashtbl.add ht_e_n encoding name;
+        })
+      Utf8.utf8_list;
+    (ht_e_n, ht_n_e)
+};
+
 value string_ini_eq s1 i s2 =
   loop i 0 where rec loop i j =
     if j = String.length s2 then True
@@ -929,38 +963,13 @@ value string_ini_eq s1 i s2 =
 ;
 
 value particle s i =
-  string_ini_eq s i "des " || string_ini_eq s i "DES " ||
-  string_ini_eq s i "de " || string_ini_eq s i "DE " ||
-  string_ini_eq s i "du " || string_ini_eq s i "DU " ||
-  string_ini_eq s i "d'" || string_ini_eq s i "D'" ||
-  string_ini_eq s i "y " || string_ini_eq s i "Y "
-;
-
-value lowercase_name s =
-  let s = String.copy s in
-  let rec loop uncap i =
-    if i = String.length s then s
-    else do {
-      let c = s.[i] in
-      let (c, uncap) =
-        match c with
-        [ 'a'..'z' ->
-            (if uncap then c
-             else Char.chr (Char.code c - Char.code 'a' + Char.code 'A'),
-             True)
-        | 'A'..'Z' ->
-            (if not uncap then c
-             else Char.chr (Char.code c - Char.code 'A' + Char.code 'a'),
-             True)
-        | c ->
-            if Char.code c < 128 then (c, particle s (i + 1))
-            else (c, uncap) ]
-      in
-      s.[i] := c;
-      loop uncap (i + 1)
-    }
+  let particles =
+    ["af "; "d'"; "d’"; "dal "; "de "; "des "; "di "; "du "; "of ";
+     "van "; "von und zu "; "von "; "y "; "zu "; "zur ";
+     "AF "; "D'"; "D’"; "DAL "; "DE "; "DES "; "DI "; "DU "; "OF ";
+     "VAN "; "VON UND ZU "; "VON "; "Y "; "ZU "; "ZUR "]
   in
-  loop (particle s 0) 0
+  List.exists (string_ini_eq s i) particles
 ;
 
 value look_like_a_number s =
@@ -991,7 +1000,7 @@ value rec next_sep_pos s i =
 ;
 
 value public_name_word =
-  ["Ier"; "Ire"; "der"; "den"; "die"; "el"; "le"; "la"; "the"]
+  ["Ier"; "Ière"; "der"; "den"; "die"; "el"; "le"; "la"; "the"]
 ;
 
 value rec is_a_public_name s i =
@@ -1008,24 +1017,171 @@ value rec is_a_public_name s i =
     else False
 ;
 
-value lowercase_public_name s =
+value gen_lowercase_uppercase_utf8_letter lower s =
+  (* liste des code hexa correspondant à l'encodage du caractère e. *)
+  let list_of_encodings e =
+    let rec loop len e l =
+      if e = "" then l
+      else
+        let i = String.index e '/' in
+        let j =
+          try String.index_from e (i+1) '/'
+          with [ Not_found -> String.length e ]
+        in
+        let k = "0" ^ String.sub e (i+1) (j-1) in
+        loop (len+1)
+          (String.sub e j (String.length e - j)) [int_of_string k :: l]
+    in
+    let l = loop 0 e [] in
+    List.rev l
+  in
+  (* l'encodage du caractère s. *)
+  let encoding =
+    loop 0 s "" where rec loop i s e =
+      if i = String.length s then e
+      else
+        let e = e ^ Printf.sprintf "/x%x" (Char.code s.[i]) in
+        loop (i + 1) s e
+  in
+  try
+    let name = Hashtbl.find ht_e_n encoding in
+    let name =
+      if lower then Str.replace_first (Str.regexp "CAPITAL") "SMALL" name
+      else Str.replace_first (Str.regexp "SMALL") "CAPITAL" name
+    in
+    let new_encoding = Hashtbl.find ht_n_e name in
+    let (el, len) =
+      let l = list_of_encodings new_encoding in
+      (l, List.length l)
+    in
+    let s = String.create len in
+    loop 0 el s where rec loop i el s =
+      match el with
+      [ [] -> s
+      | [e :: ell] ->
+          let _s = String.set s i (Char.chr e) in
+          loop (i + 1) ell s ]
+  with [ Not_found -> s ]
+;
+
+value lowercase_utf8_letter = gen_lowercase_uppercase_utf8_letter True;
+value uppercase_utf8_letter = gen_lowercase_uppercase_utf8_letter False;
+
+value capitalize_word s =
+  let s = String.copy s in
+  copy False 0 0 (particle s 0) where rec copy special i len uncap =
+    if i = String.length s then Buff.get len
+    else
+      match s.[i] with
+      [ 'a'..'z' as c ->
+          let c =
+            if uncap then c
+            else Char.chr (Char.code c - Char.code 'a' + Char.code 'A')
+          in
+          copy False (i + 1) (Buff.store len c) True
+      | 'A'..'Z' as c ->
+          let c =
+            if not uncap then c
+            else Char.chr (Char.code c - Char.code 'A' + Char.code 'a')
+          in
+          copy False (i + 1) (Buff.store len c) True
+      | c ->
+          if Char.code c < 128 then
+            copy False (i + 1) (Buff.store len c) (particle s (i+1))
+          else
+            let nbc = Name.nbc s.[i] in
+            if nbc = 1 || nbc < 0 || i + nbc > String.length s then
+              copy False (i + 1) (Buff.store len s.[i]) True
+            else
+              let s = String.sub s i nbc in
+              let s =
+                if not uncap then uppercase_utf8_letter s
+                else lowercase_utf8_letter s
+              in
+              let (t, j) = (s, i + nbc) in
+              copy False j (Buff.mstore len t) True ]
+;
+
+value uppercase_word s =
+  let s = String.copy s in
+  copy False 0 0 (particle s 0) where rec copy special i len uncap =
+    if i = String.length s then Buff.get len
+    else
+      match s.[i] with
+      [ 'a'..'z' as c ->
+          let c =
+            if uncap then c
+            else Char.chr (Char.code c - Char.code 'a' + Char.code 'A')
+          in
+          copy False (i + 1) (Buff.store len c) uncap
+      | 'A'..'Z' as c ->
+          let c =
+            if not uncap then c
+            else Char.chr (Char.code c - Char.code 'A' + Char.code 'a')
+          in
+          copy False (i + 1) (Buff.store len c) uncap
+      | c ->
+          if Char.code c < 128 then
+            copy False (i + 1) (Buff.store len c) (particle s (i+1))
+          else
+            let nbc = Name.nbc s.[i] in
+            if nbc = 1 || nbc < 0 || i + nbc > String.length s then
+              copy False (i + 1) (Buff.store len s.[i]) False
+            else
+              let s = String.sub s i nbc in
+              let s = if uncap then s else uppercase_utf8_letter s in
+              let (t, j) = (s, i + nbc) in
+              copy False j (Buff.mstore len t) False ]
+;
+
+module Buff2 = Buff.Make (struct value buff = ref (String.create 80); end);
+
+value capitalize_name s =
+  (* On initialise le buffer à la valeur de s. *)
+  let _ = Buff2.mstore 0 s in
   loop 0 0 where rec loop len k =
     let i = next_word_pos s k in
-    if i = String.length s then Buff.get len
+    if i = String.length s then Buff2.get (String.length s)
     else
       let j = next_sep_pos s i in
       if j > i then
         let w = String.sub s i (j - i) in
         let w =
-          if is_roman_int w || List.mem w public_name_word then w
-          else String.capitalize (String.lowercase w)
+          if is_roman_int w || particle s i || List.mem w public_name_word ||
+             start_with_int w
+          then w
+          else capitalize_word w
         in
         let len =
           loop len k where rec loop len k =
-            if k = i then len else loop (Buff.store len s.[k]) (k + 1)
+            if k = i then len else loop (Buff2.store len s.[k]) (k + 1)
         in
-        loop (Buff.mstore len w) j
-      else Buff.get len
+        loop (Buff2.mstore len w) j
+      else Buff2.get len
+;
+
+value uppercase_name s =
+  (* On initialise le buffer à la valeur de s. *)
+  let _ = Buff2.mstore 0 s in
+  loop 0 0 where rec loop len k =
+    let i = next_word_pos s k in
+    if i = String.length s then Buff2.get (String.length s)
+    else
+      let j = next_sep_pos s i in
+      if j > i then
+        let w = String.sub s i (j - i) in
+        let w =
+          if is_roman_int w || particle s i || List.mem w public_name_word ||
+             start_with_int w
+          then w
+          else uppercase_word w
+        in
+        let len =
+          loop len k where rec loop len k =
+            if k = i then len else loop (Buff2.store len s.[k]) (k + 1)
+        in
+        loop (Buff2.mstore len w) j
+      else Buff2.get len
 ;
 
 value get_lev0 =
@@ -1370,8 +1526,10 @@ value rec build_remain_tags =
 value applycase_surname s =
   match case_surnames.val with
   [ NoCase -> s
-  | LowerCase -> lowercase_name s
-  | UpperCase -> String.uppercase s ]
+  | LowerCase -> capitalize_name s
+  | UpperCase ->
+      if charset.val = Utf8 then uppercase_name s
+      else String.uppercase s ]
 ;
 
 value add_indi gen r =
@@ -1389,12 +1547,13 @@ value add_indi gen r =
     match name_sons with
     [ Some n ->
         let (f, s) = parse_name (Stream.of_string n.rval) in
-        let pn = if givn = f then "" else givn in
-        let fal = [] in
+        let pn = "" in
+        let fal = if givn = f then [] else [givn] in
         let (f, fal) =
           match first_names_brackets.val with
+(*
           [ Some (' ', eb) ->
-              try
+              let first_enclosed f =
                 let j = String.index f eb in
                 let i =
                   try String.rindex_from f (j - 1) ' ' with
@@ -1402,14 +1561,26 @@ value add_indi gen r =
                 in
                 let fn = String.sub f (i + 1) (j - i - 1) in
                 let fa =
-                  String.sub f 0 j ^
+                  String.sub f 0 j ^ fn ^
                     String.sub f (j + 1) (String.length f - j - 1)
                 in
-                if fn = fa then (fn, fal) else (fn, [fa :: fal])
-              with
-              [ Not_found -> (f, fal) ]
-          | Some (bb, eb) ->
-              try
+                (fn, fa)
+              in
+              loop True f "" where rec loop first ff accu =
+                try
+                  let (fn, fa) = first_enclosed ff in
+                  let accu =
+                    if first then fn
+                    else if fn <> "" then accu ^ " " ^ fn
+                    else accu
+                  in
+                  loop False fa accu
+                with
+                [ Not_found ->
+                    if f = ff then (f, fal) else (accu, [ff :: fal]) ]
+*)
+          [ Some (bb, eb) ->
+              let first_enclosed f =
                 let i = String.index f bb in
                 let j =
                   if i + 2 >= String.length f then raise Not_found
@@ -1420,9 +1591,20 @@ value add_indi gen r =
                   String.sub f 0 i ^ fn ^
                     String.sub f (j + 1) (String.length f - j - 1)
                 in
-                if fn = fa then (fn, fal) else (fn, [fa :: fal])
-              with
-              [ Not_found -> (f, fal) ]
+                (fn, fa)
+              in
+              loop True f "" where rec loop first ff accu =
+                try
+                  let (fn, fa) = first_enclosed ff in
+                  let accu =
+                    if first then fn
+                    else if fn <> "" then accu ^ " " ^ fn
+                    else accu
+                  in
+                  loop False fa accu
+                with
+                [ Not_found ->
+                    if f = ff then (f, fal) else (accu, [ff :: fal]) ]
           | None -> (f, fal) ]
         in
         let (f, pn, fal) =
@@ -1439,14 +1621,14 @@ value add_indi gen r =
               else (fn, pn, [f :: fal])
           else (f, pn, fal)
         in
-        let f = if lowercase_first_names.val then lowercase_name f else f in
+        let f = if lowercase_first_names.val then capitalize_name f else f in
         let fal =
-          if lowercase_first_names.val then List.map lowercase_name fal
+          if lowercase_first_names.val then List.map capitalize_name fal
           else fal
         in
-        let pn = if lowercase_name pn = f then "" else pn in
+        let pn = if capitalize_name pn = f then "" else pn in
         let pn =
-          if lowercase_first_names.val then lowercase_public_name pn else pn
+          if lowercase_first_names.val then capitalize_name pn else pn
         in
         let fal =
           List.fold_right (fun fa fal -> if fa = pn then fal else [fa :: fal])
@@ -1463,6 +1645,15 @@ value add_indi gen r =
         do { incr r; (f, s, r.val, pn, fal) }
     | None -> ("?", "?", Adef.int_of_iper ip, givn, []) ]
   in
+  (* S'il y a des caractères interdits, on les supprime *)
+  let (first_name, surname) =
+    (Name.strip_c first_name ':', Name.strip_c surname ':')
+  in
+  (* Si le prénom ou le nom est vide *)
+  let (first_name, surname) =
+    (if first_name = "" then default_name.val else first_name,
+     if surname = "" then default_name.val else surname)
+  in
   let qualifier =
     match name_sons with
     [ Some n ->
@@ -1614,15 +1805,15 @@ value add_indi gen r =
           [ Some r -> strip_spaces r.rval
           | _ -> "" ]
         in
-        (Adef.codate_of_od d, p, source gen r)
-    | None -> (Adef.codate_None, "", ("", [])) ]
+        (d, p, source gen r)
+    | None -> (None, "", ("", [])) ]
   in
   let (death, death_place, (death_src, death_nt)) =
     match find_field "DEAT" r.rsons with
     [ Some r ->
         if r.rsons = [] then
           if r.rval = "Y" then (DeadDontKnowWhen, "", ("", []))
-          else (infer_death birth, "", ("", []))
+          else (infer_death birth bapt, "", ("", []))
         else
           let d =
             match find_field "DATE" r.rsons with
@@ -1638,7 +1829,7 @@ value add_indi gen r =
             | _ -> "" ]
           in
           (d, p, source gen r)
-    | None -> (infer_death birth, "", ("", [])) ]
+    | None -> (infer_death birth bapt, "", ("", [])) ]
   in
   let (burial, burial_place, (burial_src, burial_nt)) =
     let (buri, buri_place, (buri_src, buri_nt)) =
@@ -1686,6 +1877,7 @@ value add_indi gen r =
     | _ -> (buri, buri_place, (buri_src, buri_nt)) ]
   in
   let birth = Adef.codate_of_od birth in
+  let bapt = Adef.codate_of_od bapt in
   let (psources, psources_nt) =
     let (s, s_nt) = source gen r in
     if s = "" then (default_source.val, s_nt) else (s, s_nt)
@@ -1761,15 +1953,25 @@ value add_indi gen r =
 
 value add_fam_norm gen r adop_list =
   let i = fam_index gen r.rval in
-  let fath =
-    match find_field "HUSB" r.rsons with
-    [ Some r -> per_index gen r.rval
-    | None -> phony_per gen Male ]
-  in
-  let moth =
-    match find_field "WIFE" r.rsons with
-    [ Some r -> per_index gen r.rval
-    | None -> phony_per gen Female ]
+  let (fath, moth, gay) =
+    match (find_all_fields "HUSB" r.rsons, find_all_fields "WIFE" r.rsons) with
+    [ ([f1], [m1]) -> (per_index gen f1.rval, per_index gen m1.rval, False)
+    | ([f1; f2 :: []], []) ->
+        (per_index gen f1.rval, per_index gen f2.rval, True)
+    | ([], [m1; m2 :: []]) ->
+        (per_index gen m1.rval, per_index gen m2.rval, True)
+    | _ ->
+        let fath =
+          match find_field "HUSB" r.rsons with
+          [ Some r -> per_index gen r.rval
+          | None -> phony_per gen Male ]
+        in
+        let moth =
+          match find_field "WIFE" r.rsons with
+          [ Some r -> per_index gen r.rval
+          | None -> phony_per gen Female ]
+        in
+        (fath, moth, False) ]
   in
   do {
     match gen.g_per.arr.(Adef.int_of_iper fath) with
@@ -1816,7 +2018,8 @@ value add_fam_norm gen r adop_list =
     let (relation, marr, marr_place, (marr_src, marr_nt), witnesses) =
       let (relation, sons) =
         match find_field "MARR" r.rsons with
-        [ Some r -> (Married, Some r)
+        [ Some r ->
+            if gay then (NoSexesCheckMarried, Some r) else (Married, Some r)
         | None ->
             match find_field "ENGA" r.rsons with
             [ Some r -> (Engaged, Some r)
@@ -1857,7 +2060,7 @@ value add_fam_norm gen r adop_list =
             fun
             [ [] -> []
             | [ r :: asso_l ] ->
-                if find_field_with_value "RELA" "Witness" r.rsons && 
+                if find_field_with_value "RELA" "Witness" r.rsons &&
                    find_field_with_value "TYPE" "INDI" r.rsons then
                   let witness = per_index gen r.rval in
                   [ witness :: heredis_witnesses asso_l ]
@@ -1926,7 +2129,7 @@ value add_fam_norm gen r adop_list =
       | Right3 p a u ->
           let notes = gen.g_str.arr.(Adef.int_of_istr (get_notes p)) in
           let notes =
-            if notes = "" then ext_sources ^ ext_notes 
+            if notes = "" then ext_sources ^ ext_notes
             else if ext_sources = "" then notes ^ "\n" ^ ext_notes
             else notes ^ "<br>\n" ^ ext_sources ^ ext_notes
           in
@@ -2466,6 +2669,7 @@ be - First names enclosed -
    ("-no_nd", Arg.Set no_negative_dates, " \
 - No negative dates -
        Don't interpret a year preceded by a minus sign as a negative year");
+   ("-nc", Arg.Clear Db2link.do_check, "\n       No consistency check");
    ("-nopicture", Arg.Set no_picture, " \
 - Don't extract individual picture.");
    ("-udi", Arg.String set_undefined_death_interval, "\
@@ -2480,6 +2684,9 @@ x-y   - Undefined death interval -
    ("-ds", Arg.String (fun s -> default_source.val := s), " \
 - Default source -
        Set the source field for persons and families without source data");
+   ("-dn", Arg.String (fun s -> default_name.val := s), " \
+- Default name -
+       Set the first name or surname field for persons without name");
    ("-dates_dm", Arg.Unit (fun () -> month_number_dates.val := DayMonthDates),
     "\n       Interpret months-numbered dates as day/month/year");
    ("-dates_md", Arg.Unit (fun () -> month_number_dates.val := MonthDayDates),
