@@ -821,6 +821,40 @@ value simple conf =
   else print_file conf "bso.htm"
 ;
 
+value simple2 conf =
+  let ged =
+    match p_getenv conf.env "anon" with
+    [ Some f -> strip_spaces f
+    | None -> "" ]
+  in
+  let ged =
+    if Filename.check_suffix (String.lowercase ged) ".ged" then ged
+    else ""
+  in
+  let out_file =
+    match p_getenv conf.env "o" with
+    [ Some f -> strip_spaces f
+    | _ -> "" ]
+  in
+  let out_file =
+    if ged = "" then out_file
+    else if out_file = "" then out_name_of_ged ged
+    else out_file
+  in
+  let env = [("f", "on") :: conf.env] in
+  let env = list_replace "anon" ged env in
+  let conf =
+    {comm = if ged = "" then "gwc2" else "ged2gwb2";
+     env = list_replace "o" out_file env; lang = conf.lang;
+     request = conf.request; lexicon = conf.lexicon}
+  in
+  if ged <> "" && not (Sys.file_exists ged) then
+    print_file conf "err_unkn.htm"
+  else if out_file = "" then print_file conf "err_miss.htm"
+  else if not (good_name out_file) then print_file conf "err_name.htm"
+  else print_file conf "bso.htm"
+;
+
 value gwc_or_ged2gwb out_name_of_in_name conf =
   let in_file =
     match p_getenv conf.env "anon" with
@@ -1165,29 +1199,39 @@ value recover_2 conf =
   }
 ;
 
-value rm_base dir =
-  match
-    try Some (Unix.opendir dir) with [ Unix.Unix_error _ _ _ -> None ]
-  with
-  [ Some dh ->
-      let list = ref [] in
-      do {
-        try
-          while True do {
-            let file = Unix.readdir dh in
-            if file = "." || file = ".." then ()
-            else list.val := [file :: list.val]
-          }
-        with
-        [ End_of_file -> () ];
-        Unix.closedir dh;
-        List.iter (fun file -> Unix.unlink (Filename.concat dir file))
-          list.val;
-        try Unix.rmdir dir with [ Unix.Unix_error _ _ _ -> () ]
-      }
-  | _ -> () ]
+value rmdir dir =
+  (* Récupère tous les fichiers et dossier d'un dossier         *)
+  (* et renvoie la liste des dossiers et la liste des fichiers. *)
+  let read_files_folders fname =
+    let list = 
+      List.map 
+        (fun file -> Filename.concat fname file)
+        (Array.to_list (Sys.readdir fname)) 
+    in
+    List.partition Sys.is_directory list
+  in
+  (* Parcours récursif de tous les dossiers *)
+  let rec loop l folders files =
+    match l with
+    [ [] -> (folders, files)
+    | [x :: l] ->
+        let (fd, fi) = read_files_folders x in
+        let l = List.rev_append l fd in
+        let folders = List.rev_append fd folders in
+        let files = List.rev_append fi files in
+        loop l folders files ]
+  in
+  (* Toute l'arborescence de dir *)
+  let (folders, files) = loop [dir] [] [] in
+  do {
+    List.iter Unix.unlink files;
+    List.iter Unix.rmdir folders;
+    Unix.rmdir dir
+  }
 ;
 
+value rm_base dir = rmdir dir;
+
 value cleanup conf =
   let in_base =
     match p_getenv conf.env "anon" with
@@ -1678,6 +1722,7 @@ value setup_comm_ok conf =
   fun
   [ "gwsetup" -> setup_gen conf
   | "simple" -> simple conf
+  | "simple2" -> simple2 conf
   | "recover" -> recover conf
   | "recover_1" -> recover_1 conf
   | "recover_2" -> recover_2 conf
