@@ -76,10 +76,18 @@ value print_differences conf base branches p1 p2 =
       False (transl conf "number") "number"
       (fun p -> string_of_int (get_occ p));
     string_field True (transl_nth conf "image/images" 0) "image"
-      (fun p -> sou base (get_image p));
+      (fun p ->
+        let v =
+          image_and_size conf base p (limited_image_size 75 100)
+        in
+        match v with
+        [ Some (False, link, _) ->
+            "<img src=\"" ^ link ^
+              "\" style=\"max-width:75px; max-height:100px\" />"
+        | _ -> sou base (get_image p) ]);
     string_field True (transl conf "public name") "public_name"
       (fun p -> sou base (get_public_name p));
-    string_field True (transl_nth conf "occupation/occupation" 0) "occupation"
+    string_field True (transl_nth conf "occupation/occupations" 0) "occupation"
       (fun p -> sou base (get_occupation p));
     string_field False (transl conf "sex") "sex"
       (fun p ->
@@ -680,11 +688,12 @@ value print conf base =
         do {
           if changes_done then Util.commit_patches conf base else ();
           if ok then do {
-            let key =
-              (sou base (get_first_name p1), sou base (get_surname p1),
-               get_occ p1, get_key_index p1)
+            let changed =
+              let p1 = Util.string_gen_person base (gen_person_of_person p1) in
+              let p2 = Util.string_gen_person base (gen_person_of_person p2) in
+              U_Merge_person p2 p1 p1
             in
-            History.record conf base key "fp";
+            History.record conf base changed "fp";
             Update.delete_topological_sort conf base;
             print_merged conf base (List.rev rev_wl.val) p1;
           }
@@ -736,16 +745,16 @@ value print_kill_ancestors conf base =
   [ Some "yes" ->
       match find_person_in_env conf base "" with
       [ Some p ->
-          let key =
-            (sou base (get_first_name p), sou base (get_surname p),
-             get_occ p, get_key_index p)
-          in
           let nb_ind = ref 0 in
           let nb_fam = ref 0 in
           do {
             kill_ancestors conf base False p nb_ind nb_fam;
             Util.commit_patches conf base;
-            History.record conf base key "ka";
+            let changed =
+              U_Kill_ancestors
+                (Util.string_gen_person base (gen_person_of_person p))
+            in
+            History.record conf base changed "ka";
             print_killed conf base p nb_ind.val nb_fam.val;
           }
       | None -> incorrect_request conf ]
