@@ -16,14 +16,20 @@ type create_info =
     ci_death : death;
     ci_death_date : option date;
     ci_death_place : string;
+    ci_occupation : string;
     ci_public : bool }
 ;
 type create = [ Create of sex and option create_info | Link ];
 type key = (string * string * int * create * string);
 
-value infer_death conf birth =
-  match birth with
-  [ Some (Dgreg d _) ->
+value infer_death conf birth bapt =
+  match (birth, bapt) with
+  [ (Some (Dgreg d _), _) ->
+      let a = (CheckItem.time_elapsed d conf.today).year in
+      if a > 120 then OfCourseDead
+      else if a <= 80 then NotDead
+      else DontKnowIfDead
+  | (_, Some (Dgreg d _)) ->
       let a = (CheckItem.time_elapsed d conf.today).year in
       if a > 120 then OfCourseDead
       else if a <= 80 then NotDead
@@ -72,13 +78,13 @@ value print_same_name conf base p =
                [ Some p -> do {
                    print_person p;
                    let ifam = get_family p in
-                   List.iter 
+                   List.iter
                      (fun ifam -> do {
                        let fam = foi base ifam in
                        let sp = spouse (get_key_index p) fam in
                        let sp = poi base sp in
                        Wserver.wprint ", &amp; ";
-                       print_person sp } ) 
+                       print_person sp } )
                      (Array.to_list ifam);
                    Wserver.wprint "\n"
                  }
@@ -96,8 +102,8 @@ value print_return conf =
         (fun (x, v) ->
            (* Seul un textarea peut contenir des sauts de ligne. *)
            (* On remplace donc l'input par un textarea.          *)
-           if x = "notes" then
-             tag "textarea" "style=\"display:none;\" name=\"%s\"" x 
+           if x = "notes" || x = "comment" then
+             tag "textarea" "style=\"display:none;\" name=\"%s\"" x
                begin
                  Wserver.wprint "%s" (quote_escaped (decode_varenv v));
                end
@@ -221,7 +227,17 @@ value print_first_name_ref conf base p =
 
 value print_warning conf base =
   fun
-  [ BirthAfterDeath p ->
+  [ BigAgeBetweenSpouses fath moth a ->
+      do {
+        Wserver.wprint
+          (fcapitale
+             (ftransl conf
+                "the difference of age between %t and %t is quite important"))
+          (fun _ -> print_someone_strong conf base fath)
+          (fun _ -> print_someone_strong conf base moth);
+        Wserver.wprint ": %s" (Date.string_of_age conf a)
+      }
+  | BirthAfterDeath p ->
       Wserver.wprint (ftransl conf "%t died before his/her birth")
         (fun _ ->
            Printf.sprintf "%s%s" (print_someone_strong conf base p)
@@ -290,6 +306,74 @@ value print_warning conf base =
           end;
         end;
       }
+  | ChangedOrderOfMarriages p before after -> do {
+      Wserver.wprint "%s\n"
+        (capitale (transl conf "changed order of marriages"));
+      Wserver.wprint "-&gt;\n";
+      let print_list arr diff_arr =
+        Array.iteri
+          (fun i ifam ->
+             let fam = foi base ifam in
+             let sp = spouse (get_key_index p) fam in
+             let sp = poi base sp in
+             tag "li" "%s"
+               (if diff_arr.(i) then "style=\"background:pink\"" else "")
+             begin
+               print_first_name conf base p;
+               Wserver.wprint "  &amp;";
+               Wserver.wprint "%s\n"
+                 (Date.short_marriage_date_text conf base fam p sp);
+               print_someone conf base sp;
+               Wserver.wprint "\n";
+             end)
+          arr
+      in
+      let (bef_d, aft_d) = Diff.f before after in
+      tag "table" "style=\"margin:1em\"" begin
+        tag "tr" begin
+          tag "td" begin
+            tag "ul" "style=\"list-style-type:none\"" begin
+              print_list before bef_d;
+            end;
+          end;
+          tag "td" begin
+            tag "ul" "style=\"list-style-type:none\"" begin
+              print_list after aft_d;
+            end;
+          end;
+        end;
+      end
+    }
+  | CloseChildren ifam des elder x ->
+      let cpl = foi base ifam in
+      do {
+        Wserver.wprint
+          (fcapitale
+             (ftransl conf
+                "the following children of %t and %t are born very close"))
+          (fun _ ->
+             print_someone_strong conf base (poi base (get_father cpl)))
+          (fun _ ->
+             print_someone_strong conf base (poi base (get_mother cpl)));
+        Wserver.wprint ":\n";
+        tag "ul" begin
+          stag "li" begin
+            print_first_name_strong conf base elder;
+            Wserver.wprint "%s" (Date.short_dates_text conf base elder);
+          end;
+          stag "li" begin
+            print_first_name_strong conf base x;
+            Wserver.wprint "%s" (Date.short_dates_text conf base x);
+          end;
+        end;
+      }
+  | DeadOld p a ->
+      do {
+        Wserver.wprint "%s\n%s\n" (print_someone_strong conf base p)
+          (transl_nth
+             conf "died at an advanced age" (index_of_sex (get_sex p)));
+        Wserver.wprint "(%s)" (Date.string_of_age conf a);
+      }
   | DeadTooEarlyToBeFather father child ->
       Wserver.wprint
         (ftransl conf "\
@@ -340,6 +424,12 @@ value print_warning conf base =
           (transl conf "is a very young parent");
         Wserver.wprint "(%s)" (Date.string_of_age conf a);
       }
+  | ParentTooOld p a ->
+      do {
+        Wserver.wprint "%s\n%s\n" (print_someone_strong conf base p)
+          (transl conf "is a very old parent");
+        Wserver.wprint "(%s)" (Date.string_of_age conf a);
+      }
   | TitleDatesError p t ->
       Wserver.wprint
         (fcapitale (ftransl conf "%t has incorrect title dates: %t"))
@@ -358,6 +448,18 @@ value print_warning conf base =
   | UndefinedSex p ->
       Wserver.wprint (fcapitale (ftransl conf "undefined sex for %t"))
         (fun _ -> print_someone_strong conf base p)
+  | WitnessDateAfterDeath p ->
+      Wserver.wprint
+        (fcapitale (ftransl conf "%t was witness after his/her death"))
+        (fun _ ->
+           Printf.sprintf "%s%s" (print_someone_strong conf base p)
+             (Date.short_dates_text conf base p))
+  | WitnessDateBeforeBirth p ->
+      Wserver.wprint
+        (fcapitale (ftransl conf "%t was witness before his/her birth"))
+        (fun _ ->
+           Printf.sprintf "%s%s" (print_someone_strong conf base p)
+             (Date.short_dates_text conf base p))
   | YoungForMarriage p a ->
      do {
         Wserver.wprint "%s\n" (print_someone_strong conf base p);
@@ -395,8 +497,8 @@ value print_warnings conf base wl =
 (* ************************************************************************* *)
 value print_misc conf base =
   fun
-  [ MissingSources -> 
-      stag "em" begin 
+  [ MissingSources ->
+      stag "em" begin
         Wserver.wprint "%s\n" (capitale (transl conf "missing sources"));
       end ]
 ;
@@ -419,7 +521,7 @@ value print_miscs conf base ml =
     Wserver.wprint "%s\n" (capitale (transl conf "miscellaneous informations"));
     tag "ul" begin
       List.iter
-        (fun m -> 
+        (fun m ->
           do {
             html_li conf; print_misc conf base m; Wserver.wprint "\n"
           })
@@ -430,7 +532,7 @@ value print_miscs conf base ml =
 
 
 (* ************************************************************************* *)
-(*  [Fonc] print_miscs : 
+(*  [Fonc] print_miscs :
       config -> base -> (Def.warning list * Def.misc list) -> unit           *)
 (** [Description] : Affiche sous la même rubrique, la liste des warnings
                     et la liste des 'informations diverses'.
@@ -455,7 +557,7 @@ value print_warnings_and_miscs conf base (wl, ml) =
            })
         wl;
       List.iter
-        (fun m -> 
+        (fun m ->
           do {
             html_li conf; print_misc conf base m; Wserver.wprint "\n"
           })
@@ -493,9 +595,17 @@ value error_locked conf =
             List.iter
               (fun (x, v) ->
                  if x = "retry" then ()
-                 else
-                   xtag "input" "type=\"hidden\" name=\"%s\" value=\"%s\"" x
-                     (quote_escaped (decode_varenv v)))
+                 else do {
+                   (* Seul un textarea peut contenir des sauts de ligne. *)
+                   (* On remplace donc l'input par un textarea.          *)
+                   if x = "notes" then
+                     tag "textarea" "style=\"display:none;\" name=\"%s\"" x
+                       begin
+                         Wserver.wprint "%s" (quote_escaped (decode_varenv v));
+                       end
+                   else
+                     xtag "input" "type=\"hidden\" name=\"%s\" value=\"%s\"" x
+                       (quote_escaped (decode_varenv v))})
               (conf.henv @ conf.env);
             (* just to see in the traces... *)
             xtag "input" "type=\"hidden\" name=\"retry\" value=\"%s\""
@@ -722,8 +832,16 @@ value print_create_conflict conf base p var =
     tag "form" "method=\"post\" action=\"%s\"" conf.command begin
       List.iter
         (fun (x, v) ->
-           xtag "input" "type=\"hidden\" name=\"%s\" value=\"%s\"" x
-             (quote_escaped (decode_varenv v)))
+           (* Seul un textarea peut contenir des sauts de ligne. *)
+           (* On remplace donc l'input par un textarea.          *)
+           if x = "notes" then
+             tag "textarea" "style=\"display:none;\" name=\"%s\"" x
+               begin
+                 Wserver.wprint "%s" (quote_escaped (decode_varenv v));
+               end
+           else
+             xtag "input" "type=\"hidden\" name=\"%s\" value=\"%s\"" x
+               (quote_escaped (decode_varenv v)))
         (conf.henv @ conf.env);
       xtag "input" "type=\"hidden\" name=\"field\" value=\"%s\"" var;
       xtag "input" "type=\"hidden\" name=\"free_occ\" value=\"%d\"" free_n;
@@ -808,7 +926,12 @@ value insert_person conf base src new_persons (f, s, o, create, var) =
               {ci_death = (DeadDontKnowWhen | NotDead as dead);
                ci_death_date = None; ci_death_place = dpl} ->
                 (dead, dpl)
-            | _ -> (infer_death conf birth, "") ]
+            | _ -> (infer_death conf birth baptism, "") ]
+          in
+          let occupation =
+            match info with
+            [ Some { ci_occupation = occupation } -> occupation
+            | _ -> "" ]
           in
           let access =
             match info with
@@ -822,7 +945,8 @@ value insert_person conf base src new_persons (f, s, o, create, var) =
              first_names_aliases = []; surnames_aliases = [];
              public_name = empty_string; qualifiers = []; aliases = [];
              titles = []; rparents = []; related = [];
-             occupation = empty_string; sex = sex; access = access;
+             occupation = Gwdb.insert_string base occupation;
+             sex = sex; access = access;
              birth = Adef.codate_of_od birth;
              birth_place = Gwdb.insert_string base birth_place;
              birth_src = empty_string; baptism = Adef.codate_of_od baptism;
@@ -883,7 +1007,7 @@ value rec update_conf_env field p occ o_env n_env =
         update_conf_env field p occ rest n_env
       else update_conf_env field p occ rest [head :: n_env] ]
 ;
-  
+
 value update_conf_create conf =
   let field =
     match p_getenv conf.env "field" with
