@@ -17,6 +17,7 @@ type create_info =
       ci_death : death;
       ci_death_date : option date;
       ci_death_place : string;
+      ci_occupation : string;
       ci_public : bool }
 ;
 
@@ -103,7 +104,10 @@ value obsolete version var new_var r =
 value bool_val x = VVbool x;
 value str_val x = VVstring x;
 
-value rec eval_var conf base env (fam, cpl, des) loc =
+value rec eval_var conf base env (fam, cpl, des) loc sl =
+  try eval_special_var conf base (fam, cpl, des) sl with
+  [ Not_found -> eval_simple_var conf base env (fam, cpl, des) loc sl ]
+and eval_simple_var conf base env (fam, cpl, des) loc =
   fun
   [ ["bvar"; v] ->
       try VVstring (List.assoc v conf.base_env) with
@@ -257,7 +261,7 @@ and eval_parent conf base env k =
       str_val s
   | sl -> eval_key k sl ]
 and eval_key (fn, sn, oc, create, var) =
-  fun 
+  fun
   [ ["create"] -> str_val (if create <> Update.Link then "create" else "link")
   | ["create"; s] -> str_val (eval_create create s)
   | ["first_name"] -> str_val (quote_escaped fn)
@@ -307,7 +311,7 @@ and eval_create c =
       | _ -> "" ]
   | "death_day" ->
       match c with
-      [ Update.Create _ 
+      [ Update.Create _
           (Some {ci_death_date = Some (Dgreg dmy Dfrench)})
         ->
           let dmy = Calendar.french_of_gregorian dmy in
@@ -344,6 +348,11 @@ and eval_create c =
           | NotDead -> "-"
           | _ -> "" ]
       | _ -> "" ]
+  | "occupation" ->
+      match c with
+      [ Update.Create _ (Some {ci_occupation = occupation}) ->
+          quote_escaped occupation
+      | _ -> "" ]
   | "sex" ->
       match c with
       [ Update.Create Male _ -> "male"
@@ -366,6 +375,17 @@ and eval_relation_kind =
   | NoSexesCheckNotMarried -> "nsck"
   | NoSexesCheckMarried -> "nsckm"
   | NoMention -> "no_ment" ]
+and eval_special_var conf base p =
+  fun
+  [ ["include_perso_header"] ->
+      match p_getint conf.env "ip" with
+      [ Some i -> do {
+          let p = poi base (Adef.iper_of_int i) in
+          Perso.interp_templ_with_menu (fun _ -> ()) "perso_header" conf base p;
+          VVstring ""
+        }
+      | None -> VVstring "" ]
+  | _ -> raise Not_found ]
 and eval_int_env var env =
   match get_env var env with
   [ Vint x -> str_val (string_of_int x)
@@ -426,7 +446,12 @@ value print_del1 conf base ifam =
     Wserver.wprint "%s" (capitale (transl_decline conf "delete" s))
   in
   do {
-    header conf title;
+    let p =
+      match p_getint conf.env "ip" with
+      [ Some ip -> poi base (Adef.iper_of_int ip)
+      | None -> Gwdb.empty_person base (Adef.iper_of_int (-1)) ]
+    in
+    Perso.interp_notempl_with_menu title "perso_header" conf base p;
     print_link_to_welcome conf True;
     Wserver.wprint "\n";
     tag "form" "method=\"post\" action=\"%s\"" conf.command begin
@@ -455,7 +480,7 @@ value print_inv1 conf base p ifam1 ifam2 =
   let cpl1 = foi base ifam1 in
   let cpl2 = foi base ifam2 in
   do {
-    header conf title;
+    Perso.interp_notempl_with_menu title "perso_header" conf base p;
     Wserver.wprint "%s:"
       (capitale (transl conf "invert the order of the following families"));
     tag "ul" begin
@@ -580,3 +605,88 @@ value print_inv conf base =
       | _ -> incorrect_request conf ]
   | _ -> incorrect_request conf ]
 ;
+
+value change_order conf base ip u ifam n =
+  let rec loop i =
+    fun
+    [ [] -> if i = n then [ifam] else []
+    | [fam :: faml] ->
+        if ifam = fam then
+          if i = n then [ifam :: loop (i+1) [fam :: faml]]
+          else loop i faml
+        else
+          if i = n then [ifam :: loop (i+1) [fam :: faml]]
+          else [fam :: loop (i+1) faml] ]
+  in
+  loop 1 (Array.to_list (get_family u))
+;
+
+value print_change_order conf base =
+  match
+    (p_getint conf.env "i", p_getint conf.env "f", p_getint conf.env "n")
+  with
+  [ (Some ip, Some ifam, Some n) -> do {
+      let p = poi base (Adef.iper_of_int ip) in
+      let print_list arr diff_arr =
+        Array.iteri
+          (fun i ifam ->
+             let fam = foi base ifam in
+             let sp = spouse (get_key_index p) fam in
+             let sp = poi base sp in
+             tag "li" "%s"
+               (if diff_arr.(i) then "style=\"background:pink\"" else "")
+             begin
+               Wserver.wprint "%s%s" (p_first_name base p)
+                 (if get_occ p = 0 then "" else "." ^ string_of_int (get_occ p));
+               Wserver.wprint "  &amp;";
+               Wserver.wprint "%s\n"
+                 (Date.short_marriage_date_text conf base fam p sp);
+               Wserver.wprint "%s%s %s" (p_first_name base sp)
+                 (if get_occ sp = 0 then "" else "." ^ string_of_int (get_occ sp))
+                 (p_surname base sp);
+               Wserver.wprint "\n";
+             end)
+          arr
+      in
+      let after =
+        change_order conf base (get_key_index p) p (Adef.ifam_of_int ifam) n
+      in
+      let (before, after) = (get_family p, Array.of_list after) in
+      let (bef_d, aft_d) = Diff.f before after in
+      let title _ =
+        Wserver.wprint "%s" (capitale (transl_decline conf "invert" ""))
+      in
+      Perso.interp_templ_with_menu title "perso_header" conf base p;
+      Wserver.wprint "%s:"
+        (capitale (transl conf "invert the order of the following families"));
+      tag "table" "style=\"margin:1em\"" begin
+        tag "tr" begin
+          tag "td" begin
+            tag "ul" "style=\"list-style-type:none\"" begin
+              print_list before bef_d;
+            end;
+          end;
+          tag "td" begin
+            tag "ul" "style=\"list-style-type:none\"" begin
+              print_list after aft_d;
+            end;
+          end;
+        end;
+      end;
+      tag "form" "method=\"post\" action=\"%s\"" conf.command begin
+        tag "p" begin
+          Util.hidden_env conf;
+          xtag "input" "type=\"hidden\" name=\"i\" value=\"%d\"" ip;
+          xtag "input" "type=\"hidden\" name=\"f\" value=\"%d\"" ifam;
+          xtag "input" "type=\"hidden\" name=\"n\" value=\"%d\"" n;
+          xtag "input" "type=\"hidden\" name=\"m\" value=\"CHG_FAM_ORD_OK\"";
+        end;
+        tag "p" begin
+          xtag "input" "type=\"submit\" value=\"Ok\"";
+        end;
+      end;
+      Wserver.wprint "\n";
+      trailer conf
+    }
+  | _ -> incorrect_request conf ]
+;
