@@ -134,10 +134,10 @@ value reconstitute_relation_parent conf var key sex =
       let (fn, sn) =
         let contain_fn = String.contains fn in
         let contain_sn = String.contains sn in
-        if (List.exists contain_fn Name.forbidden_char) || 
+        if (List.exists contain_fn Name.forbidden_char) ||
            (List.exists contain_sn Name.forbidden_char) then
           do {
-            removed_string.val := 
+            removed_string.val :=
               [(Name.purge fn ^ " " ^ Name.purge sn) :: removed_string.val];
             (Name.purge fn, Name.purge sn)
           }
@@ -182,7 +182,7 @@ value rec reconstitute_relations conf ext cnt =
   | _ -> ([], ext) ]
 ;
 
-value reconstitute_death conf birth death_place burial burial_place =
+value reconstitute_death conf birth baptism death_place burial burial_place =
   let d = Update.reconstitute_date conf "death" in
   let dr =
     match p_getenv conf.env "death_reason" with
@@ -198,13 +198,13 @@ value reconstitute_death conf birth death_place burial burial_place =
       if death_place <> "" || burial <> UnknownBurial || burial_place <> "" ||
          dr <> Unspecified then
         DeadDontKnowWhen
-      else Update.infer_death conf birth
+      else Update.infer_death conf birth baptism
   | "DeadYoung" when d = None -> DeadYoung
   | "DontKnowIfDead" when d = None -> DontKnowIfDead
   | "NotDead" -> NotDead
   | "OfCourseDead" when d = None -> OfCourseDead
-  | _ ->   
-      match d with 
+  | _ ->
+      match d with
       [ Some d -> Death dr (Adef.cdate_of_date d)
       | _ -> DeadDontKnowWhen ] ]
 ;
@@ -237,7 +237,7 @@ value reconstitute_person conf =
     if (List.exists contain_fn Name.forbidden_char) ||
        (List.exists contain_sn Name.forbidden_char) then
       do {
-        removed_string.val := 
+        removed_string.val :=
           [(Name.purge first_name ^ " " ^ Name.purge surname) :: removed_string.val];
         (Name.purge first_name, Name.purge surname)
       }
@@ -275,12 +275,14 @@ value reconstitute_person conf =
   in
   let birth = Update.reconstitute_date conf "birth" in
   let birth_place = no_html_tags (only_printable (get conf "birth_place")) in
-  let bapt = Adef.codate_of_od (Update.reconstitute_date conf "bapt") in
+  let bapt = Update.reconstitute_date conf "bapt" in
   let bapt_place = no_html_tags (only_printable (get conf "bapt_place")) in
   let burial_place = no_html_tags (only_printable (get conf "burial_place")) in
   let burial = reconstitute_burial conf burial_place in
   let death_place = no_html_tags (only_printable (get conf "death_place")) in
-  let death = reconstitute_death conf birth death_place burial burial_place in
+  let death =
+    reconstitute_death conf birth bapt death_place burial burial_place
+  in
   let death_place =
     match death with
     [ Death _ _ | DeadYoung | DeadDontKnowWhen -> death_place
@@ -304,8 +306,8 @@ value reconstitute_person conf =
      rparents = rparents; occupation = occupation; related = []; sex = sex;
      access = access; birth = Adef.codate_of_od birth;
      birth_place = birth_place;
-     birth_src = only_printable (get conf "birth_src"); baptism = bapt;
-     baptism_place = bapt_place;
+     birth_src = only_printable (get conf "birth_src");
+     baptism = Adef.codate_of_od bapt; baptism_place = bapt_place;
      baptism_src = only_printable (get conf "bapt_src"); death = death;
      death_place = death_place;
      death_src = only_printable (get conf "death_src"); burial = burial;
@@ -328,8 +330,8 @@ value check_person conf base p =
 value error_person conf base p err =
   let title _ = Wserver.wprint "%s" (capitale (transl conf "error")) in
   do {
-    rheader conf title; 
-    Wserver.wprint "%s\n" (capitale err); 
+    rheader conf title;
+    Wserver.wprint "%s\n" (capitale err);
     Update.print_return conf;
     trailer conf;
     raise Update.ModErr
@@ -631,11 +633,11 @@ value print_mod_ok conf base wl p =
     if List.length removed_string.val > 0 then
       do {
          Wserver.wprint "<h3 class=\"error\">" ;
-         Wserver.wprint 
-           (fcapitale (ftransl conf "%s forbidden char")) 
-           (List.fold_left 
-              (fun acc c -> acc ^ "'" ^ Char.escaped c ^ "' ") 
-              " " 
+         Wserver.wprint
+           (fcapitale (ftransl conf "%s forbidden char"))
+           (List.fold_left
+              (fun acc c -> acc ^ "'" ^ Char.escaped c ^ "' ")
+              " "
               Name.forbidden_char);
          Wserver.wprint "</h3>\n" ;
          List.iter (Wserver.wprint "<p>%s</p>") removed_string.val
@@ -646,15 +648,15 @@ value print_mod_ok conf base wl p =
     [ [] -> ()
     | _ ->
         do {
-          tag "p" 
+          tag "p"
             begin
               Wserver.wprint "%s, %s %s %s :"
                 (capitale (transl_nth conf "relation/relations" 0))
                 (transl conf "first name missing")
-                (transl conf "or") 
+                (transl conf "or")
                 (transl conf "surname missing") ;
               tag "ul" begin
-                List.iter 
+                List.iter
                   (fun s -> do {stag "li" begin Wserver.wprint "%s" s; end;})
                   deleted_relation.val;
               end;
@@ -780,10 +782,10 @@ value print_add o_conf base =
           let (p, a) = effective_add conf base sp in
           let u = {family = get_family (poi base p.key_index)} in
           let wl = all_checks_person conf base p a u in
-          let k = (sp.first_name, sp.surname, sp.occ, p.key_index) in
           do {
             Util.commit_patches conf base;
-            History.record conf base k "ap";
+            let changed = U_Add_person (Util.string_gen_person base p) in
+            History.record conf base changed "ap";
             print_add_ok conf base wl p;
           } ]
     }
@@ -799,8 +801,8 @@ value print_del conf base =
       let fn = sou base (get_first_name p) in
       let sn = sou base (get_surname p) in
       let occ = get_occ p in
-      let k = (fn, sn, occ, ip) in
       let old_related = get_related p in
+      let op = Util.string_gen_person base (gen_person_of_person p) in
       update_relations_of_related base ip old_related;
       let warning _ = () in
       let p = effective_del conf base warning p in
@@ -808,7 +810,8 @@ value print_del conf base =
       delete_key base fn sn occ;
       Notes.update_notes_links_db conf (NotesLinks.PgInd p.key_index) "";
       Util.commit_patches conf base;
-      History.record conf base k "dp";
+      let changed = U_Delete_person op in
+      History.record conf base changed "dp";
       print_del_ok conf base [];
     }
   | _ -> incorrect_request conf ]
@@ -837,32 +840,21 @@ value print_mod_aux conf base callback =
   [ Update.ModErr -> () ]
 ;
 
-value get_key conf base =
-  let ip = 
-    match p_getint conf.env "i" with
-    [ Some i -> i
-    | _ -> -1 ]
-  in
-  let p = poi base (Adef.iper_of_int ip) in
-  let fn = sou base (Gwdb.get_first_name p) in
-  let sn = sou base (Gwdb.get_surname p) in
-  let occ = Gwdb.get_occ p in
-  (get_key_index p, (fn, sn, occ))
-;
-
-value notify_change_key conf base (_, (ofn, osn, oocc)) (_, (fn, sn, occ)) =
-  let old_key = default_image_name_of_key ofn osn oocc in
-  let new_key = default_image_name_of_key fn sn occ in
-  if old_key <> new_key then
-    History.record_key conf base old_key new_key
-  else ()
-;
-
 value print_mod o_conf base =
   (* Attention ! On pense à remettre les compteurs à *)
   (* zéro pour la détection des caractères interdits *)
   let () = removed_string.val := [] in
-  let old_key = get_key o_conf base in
+  let o_p =
+    match p_getint o_conf.env "i" with
+    [ Some ip ->
+        Util.string_gen_person
+          base
+          (gen_person_of_person (poi base (Adef.iper_of_int ip)))
+    | None ->
+        Util.string_gen_person
+          base
+          (gen_person_of_person (poi base (Adef.iper_of_int (-1)))) ]
+  in
   let conf = Update.update_conf o_conf in
   let callback sp = do {
     let p = effective_mod conf base sp in
@@ -888,12 +880,9 @@ value print_mod o_conf base =
       let a = {parents = get_parents a; consang = get_consang a} in
       all_checks_person conf base p a u
     in
-    let k = (sp.first_name, sp.surname, sp.occ, sp.key_index) in
     Util.commit_patches conf base;
-    History.record conf base k "mp";
-    let new_key = (sp.key_index, (sp.first_name, sp.surname, sp.occ)) in
-    update_gwf_sosa conf base new_key;
-    notify_change_key conf base old_key new_key;
+    let changed = U_Modify_person o_p (Util.string_gen_person base p) in
+    History.record conf base changed "mp";
     if not (is_quest_string p.surname) &&
        not (is_quest_string p.first_name) &&
        not (is_old_person conf p)
